/**
 ****************************************************************************************
 *
 * @file app.c
 *
 * @brief Application entry point
 *
 * Copyright (C) RivieraWaves 2009-2015
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup APP
 * @{
 ****************************************************************************************
 */

/*
 * INCLUDE FILES
 ****************************************************************************************
 */

#include "rwip_config.h"             // SW configuration

#if (BLE_APP_PRESENT)
#include <string.h>
//#include "rwapp_config.h"
#include "app_task.h"                // Application task Definition
#include "app.h"                     // Application Definition
#include "gap.h"                     // GAP Definition
#include "gapm_task.h"               // GAP Manager Task API
#include "gapc_task.h"               // GAP Controller Task API

#include "co_bt.h"                   // Common BT Definition
#include "co_math.h"                 // Common Maths Definition
#include "ke_timer.h"
#include "app_fff0.h"                 // Application security Definition
#include "app_dis.h"                 // Device Information Service Application Definitions
#include "app_batt.h"                // Battery Application Definitions
#include "app_oads.h"                 // Application oads Definition
#if (NVDS_SUPPORT)
#include "nvds.h"                    // NVDS Definitions
#endif
#include "rf.h"
#include "uart.h"
#include "adc.h"
#include "gpio.h"
#include "wdt.h"
#include "rtc.h"

#if (BLE_HT_THERMOM)
#include "app_htpt.h"                // Health Thermometer Application Definitions
#endif
#if (BLE_BP_SENSOR)
#include "app_blps.h"                // Blood Pressure Application Definitions
#endif
#if (BLE_HR_SENSOR)
#include "app_hrps.h"             
#endif
#if (BLE_GL_SENSOR)
#include "app_glps.h"             
#endif
#if (BLE_WSC_SERVER)
#include "app_wscs.h"             
#endif

#include "user_config.h"
#include "flash.h"
#if(UART2_DRIVER)
	#include "uart2.h"
#endif

#include "app_sec.h"			////
/*
 * DEFINES
 ****************************************************************************************
 */
#define APP_DEVICE_NAME_LENGTH_MAX      (18)


/*
 * TYPE DEFINITIONS
 ****************************************************************************************
 */

typedef void (*appm_add_svc_func_t)(void);

/*
 * ENUMERATIONS
 ****************************************************************************************
 */

/// List of service to add in the database
enum appm_svc_list
{
#if (BLE_WSC_SERVER)    
    APPM_SVC_WSCS,
#endif    
#if (BLE_GL_SENSOR)    
    APPM_SVC_GLPS,
#endif    
#if (BLE_HR_SENSOR)    
    APPM_SVC_HRPS,
#endif    
#if (BLE_BP_SENSOR)    
    APPM_SVC_BLPS,
#endif    
#if (BLE_HT_THERMOM)    
    APPM_SVC_HTPT,
#endif    
#if (BLE_FFF0_SERVER)
    APPM_SVC_FFF0,
#endif
    APPM_SVC_DIS,
#if (BLE_BATT_SERVER)    
    APPM_SVC_BATT,
#endif    
    APPM_SVC_OADS,
    APPM_SVC_LIST_STOP ,
};

/*
 * LOCAL VARIABLES DEFINITIONS
 ****************************************************************************************
 */

/// Application Task Descriptor
static const struct ke_task_desc TASK_DESC_APP = {NULL, &appm_default_handler,
                                                  appm_state, APPM_STATE_MAX, APP_IDX_MAX};

/// List of functions used to create the database
static const appm_add_svc_func_t appm_add_svc_func_list[APPM_SVC_LIST_STOP] =
{
#if (BLE_WSC_SERVER)    
    (appm_add_svc_func_t)app_wscs_add_ws,
#endif    
#if (BLE_GL_SENSOR)    
    (appm_add_svc_func_t)app_glps_add_gl,
#endif    
#if (BLE_HR_SENSOR)    
    (appm_add_svc_func_t)app_hrps_add_hr,
#endif    
#if (BLE_BP_SENSOR)    
    (appm_add_svc_func_t)app_blps_add_bp,
#endif    
#if (BLE_HT_THERMOM)    
    (appm_add_svc_func_t)app_htpt_add_ht,
#endif    
#if (BLE_FFF0_SERVER)
    (appm_add_svc_func_t)app_fff0_add_fff0s,
#endif
    (appm_add_svc_func_t)app_dis_add_dis,
#if (BLE_BATT_SERVER)    
    (appm_add_svc_func_t)app_batt_add_bas,
#endif    
    (appm_add_svc_func_t)app_oad_add_oads,
};

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// Application Environment Structure
struct app_env_tag app_env;

#if NVR8080_RENAME == 1
uint8_t app_nvr_device_name[APP_DEVICE_NAME_LENGTH_MAX];
uint8_t app_nvr_name_len = 0;
#endif 

/*
 * FUNCTION DEFINITIONS
 ****************************************************************************************
 */

void appm_init()
{
    // Reset the application manager environment
    memset(&app_env, 0, sizeof(app_env));

    // Create APP task
    ke_task_create(TASK_APP, &TASK_DESC_APP);

    // Initialize Task state
    ke_state_set(TASK_APP, APPM_INIT);

#if NVR8080_RENAME == 1  
    uint8_t i;
    flash_read(FLASH_SPACE_TYPE_NVR, 0x8082, APP_DEVICE_NAME_LENGTH_MAX, &app_nvr_device_name[0]);        
    for(i=0;i<APP_DEVICE_NAME_LENGTH_MAX;i++)
    {
        if(app_nvr_device_name[i]==0xff || app_nvr_device_name[i]==0 )
        {   
            app_nvr_device_name[i]=0;
            break;
        }    
    }    
    app_nvr_name_len = i;
    
    if(app_nvr_name_len!=0)
    {
        app_env.dev_name_len = app_nvr_name_len;
        memcpy(&app_env.dev_name[0], app_nvr_device_name, app_nvr_name_len);        
    }
    else 
#endif  
    {
        app_env.dev_name_len = sizeof(APP_DFLT_DEVICE_NAME);
        memcpy(&app_env.dev_name[0], APP_DFLT_DEVICE_NAME, sizeof(APP_DFLT_DEVICE_NAME));
    }
    
#if (NVDS_SUPPORT)
	  uint8_t key_len = KEY_LEN;
    if (nvds_get(NVDS_TAG_LOC_IRK, &key_len, app_env.loc_irk) != NVDS_OK)
#endif        
    {
        uint8_t counter;

        // generate a new IRK
        for (counter = 0; counter < KEY_LEN; counter++)
        {
            app_env.loc_irk[counter]    = (uint8_t)co_rand_word();
        }
#if (NVDS_SUPPORT)
        // Store the generated value in NVDS
        if (nvds_put(NVDS_TAG_LOC_IRK, KEY_LEN, (uint8_t *)&app_env.loc_irk) != NVDS_OK)
        {
            ASSERT_INFO(0, NVDS_TAG_LOC_IRK, 0);
        }
#endif        
    }


    /*------------------------------------------------------
     * INITIALIZE ALL MODULES
     *------------------------------------------------------*/

    // Device Information Module
    app_dis_init();

#if (BLE_BATT_SERVER) // Battery Module
    app_batt_init();	
#endif	

    app_oads_init();	
    
#if (BLE_HT_THERMOM)    
    app_htpt_init();
#endif
#if (BLE_BP_SENSOR) 
    app_blps_init();
#endif	    
#if (BLE_HR_SENSOR) 
    app_hrps_init();
#endif	    
#if (BLE_GL_SENSOR) 
    app_glps_init();
#endif	    
#if (BLE_WSC_SERVER) 
    app_wscs_init();
#endif	    
    
	app_sec_init();
}

bool appm_add_svc(void)
{
    // Indicate if more services need to be added in the database
    bool more_svc = false;

    // Check if another should be added in the database
    if (app_env.next_svc != APPM_SVC_LIST_STOP)
    {
        ASSERT_INFO(appm_add_svc_func_list[app_env.next_svc] != NULL, app_env.next_svc, 1);

        // Call the function used to add the required service
        appm_add_svc_func_list[app_env.next_svc]();

        // Select following service to add
        app_env.next_svc++;
        more_svc = true;
    }

    return more_svc;
}


/*豸ϿӺ*/
void appm_disconnect(void)
{
    struct gapc_disconnect_cmd *cmd = KE_MSG_ALLOC(GAPC_DISCONNECT_CMD,
                                                   KE_BUILD_ID(TASK_GAPC, app_env.conidx), TASK_APP,
                                                   gapc_disconnect_cmd);

    cmd->operation = GAPC_DISCONNECT;
    cmd->reason    = CO_ERROR_REMOTE_USER_TERM_CON;

    // Send the message
    ke_msg_send(cmd);
}

void appm_start_advertising(void)
{	
    // Check if the advertising procedure is already is progress
    if (ke_state_get(TASK_APP) == APPM_READY)
    {				
        // Prepare the GAPM_START_ADVERTISE_CMD message
        struct gapm_start_advertise_cmd *cmd = KE_MSG_ALLOC(GAPM_START_ADVERTISE_CMD,
                                                            TASK_GAPM, TASK_APP,
                                                            gapm_start_advertise_cmd);

        cmd->op.addr_src    = GAPM_STATIC_ADDR;
        cmd->channel_map    = APP_ADV_CHMAP;
        cmd->intv_min 		= APP_ADV_INT_MIN;
        cmd->intv_max 		= APP_ADV_INT_MAX;	
        cmd->op.code        = GAPM_ADV_UNDIRECT;
		
        cmd->info.host.mode = GAP_GEN_DISCOVERABLE;

 		/*-----------------------------------------------------------------------------------
         * Set the Advertising Data and the Scan Response Data
         *---------------------------------------------------------------------------------*/
        // Flag value is set by the GAP
        cmd->info.host.adv_data_len       = ADV_DATA_LEN;
        cmd->info.host.scan_rsp_data_len  = SCAN_RSP_DATA_LEN;

#if (NVDS_SUPPORT)
        // Advertising Data
        if(nvds_get(NVDS_TAG_APP_BLE_ADV_DATA, &cmd->info.host.adv_data_len,
                    &cmd->info.host.adv_data[0]) != NVDS_OK)
#endif                    
        {
            //cmd->info.host.adv_data_len = 0;

            cmd->info.host.adv_data[0] = 2;// Length of ad type flags
            cmd->info.host.adv_data[1] = GAP_AD_TYPE_FLAGS;
            cmd->info.host.adv_data[2] = GAP_BR_EDR_NOT_SUPPORTED;
            // set mode in ad_type
            switch(cmd->info.host.mode)
            {
                // General discoverable mode
                case GAP_GEN_DISCOVERABLE:
                {
                    cmd->info.host.adv_data[2] |= GAP_LE_GEN_DISCOVERABLE_FLG;
                }
                break;
                // Limited discoverable mode
                case GAP_LIM_DISCOVERABLE:
                {
                    cmd->info.host.adv_data[2] |= GAP_LE_LIM_DISCOVERABLE_FLG;
                }
                break;
                default: break; // do nothing
            }
            cmd->info.host.adv_data_len=3;
            //Add list of UUID and appearance						
        #if (BLE_GL_SENSOR)
			memcpy(&cmd->info.host.adv_data[cmd->info.host.adv_data_len],
                   APP_GLP_ADV_DATA_UUID, APP_GLP_ADV_DATA_UUID_LEN);
            cmd->info.host.adv_data_len += APP_GLP_ADV_DATA_UUID_LEN;
		#endif	////#if (BLE_GL_SENSOR)
            memcpy(&cmd->info.host.adv_data[cmd->info.host.adv_data_len],
                   APP_FFF0_ADV_DATA_UUID, APP_FFF0_ADV_DATA_UUID_LEN);
            cmd->info.host.adv_data_len += APP_FFF0_ADV_DATA_UUID_LEN;
        }

		//  Device Name Length
        uint8_t device_name_length;
        uint8_t device_name_avail_space;
        uint8_t device_name_temp_buf[APP_DEVICE_NAME_LENGTH_MAX];

        // Get remaining space in the Advertising Data - 2 bytes are used for name length/flag
        device_name_avail_space = ADV_DATA_LEN  - cmd->info.host.adv_data_len - 2;

        // Check if data can be added to the Advertising data
        if (device_name_avail_space > 2)
        {
						
            device_name_length = NVDS_LEN_DEVICE_NAME;
#if (NVDS_SUPPORT)            
            if (nvds_get(NVDS_TAG_DEVICE_NAME, &device_name_length,
                         &device_name_temp_buf[0]) != NVDS_OK)
#endif                         
            {
            #if NVR8080_RENAME == 1    
                if(app_nvr_name_len!=0)
                {
                    device_name_length = app_nvr_name_len;
                    memcpy(&device_name_temp_buf[0], app_nvr_device_name, app_nvr_name_len);        
                }
                else 
            #endif 
                {
                    device_name_length = strlen(APP_DFLT_DEVICE_NAME);
                    // Get default Device Name (No name if not enough space)
                    memcpy(&device_name_temp_buf[0], APP_DFLT_DEVICE_NAME, device_name_length);
                }   
            }
							
	     	if(device_name_length > 0)
            {
                // Check available space
                device_name_length = co_min(device_name_length, device_name_avail_space);
                cmd->info.host.adv_data[cmd->info.host.adv_data_len]     = device_name_length + 1;
								 
                // Fill Device Name Flag
                cmd->info.host.adv_data[cmd->info.host.adv_data_len + 1] = '\x09';
                // Copy device name
                memcpy(&cmd->info.host.adv_data[cmd->info.host.adv_data_len + 2],
                device_name_temp_buf, device_name_length);

                // Update Advertising Data Length
                cmd->info.host.adv_data_len += (device_name_length + 2);					
            }
          
        }
#if (NVDS_SUPPORT)
        // Scan Response Data
        if(nvds_get(NVDS_TAG_APP_BLE_SCAN_RESP_DATA, &cmd->info.host.scan_rsp_data_len,
                    &cmd->info.host.scan_rsp_data[0]) != NVDS_OK)
#endif                    
        {            
            cmd->info.host.scan_rsp_data_len = 0;            
            #if NVR8080_RENAME == 1    
            if(app_nvr_name_len!=0)
            {
                cmd->info.host.scan_rsp_data[cmd->info.host.scan_rsp_data_len++] = app_nvr_name_len+2;
                cmd->info.host.scan_rsp_data[cmd->info.host.scan_rsp_data_len++] = 0x08;
                memcpy(&cmd->info.host.scan_rsp_data[cmd->info.host.scan_rsp_data_len],
                    app_nvr_device_name, app_nvr_name_len);       
                cmd->info.host.scan_rsp_data_len += app_nvr_name_len;
            }
            else  
            #endif     
            {
                memcpy(&cmd->info.host.scan_rsp_data[cmd->info.host.scan_rsp_data_len],
                    APP_SCNRSP_DATA, APP_SCNRSP_DATA_LEN);
                cmd->info.host.scan_rsp_data_len += APP_SCNRSP_DATA_LEN;
            }    
        }

        // Send the message
        ke_msg_send(cmd);
				DBG_PRINTF("appm start advertising\r\n");

		wdt_enable(0x3fff);

        // Set the state of the task to APPM_ADVERTISING
        ke_state_set(TASK_APP, APPM_ADVERTISING);	

		//ke_timer_set(APP_PERIOD_TIMER, TASK_APP, 1500);	
    }
	
    // else ignore the request
}


/* 豸ֹͣ㲥*/
void appm_stop_advertising(void)
{
    if (ke_state_get(TASK_APP) == APPM_ADVERTISING)
    {
        // Go in ready state
        ke_state_set(TASK_APP, APPM_READY);

        // Prepare the GAPM_CANCEL_CMD message
        struct gapm_cancel_cmd *cmd = KE_MSG_ALLOC(GAPM_CANCEL_CMD,
                                                   TASK_GAPM, TASK_APP,
                                                   gapm_cancel_cmd);
        cmd->operation = GAPM_CANCEL;

        // Send the message
        ke_msg_send(cmd);

		wdt_disable_flag = 1;
    }
    // else ignore the request
}



void appm_update_param(struct gapc_conn_param *conn_param)
{
    // Prepare the GAPC_PARAM_UPDATE_CMD message
    struct gapc_param_update_cmd *cmd = KE_MSG_ALLOC(GAPC_PARAM_UPDATE_CMD,
                                                     KE_BUILD_ID(TASK_GAPC, app_env.conidx), TASK_APP,
                                                     gapc_param_update_cmd);

    cmd->operation  = GAPC_UPDATE_PARAMS;
    cmd->intv_min   = conn_param->intv_min;
    cmd->intv_max   = conn_param->intv_max;
    cmd->latency    = conn_param->latency;
    cmd->time_out   = conn_param->time_out;

    // not used by a slave device
    cmd->ce_len_min = 0xFFFF;
    cmd->ce_len_max = 0xFFFF;
		
    DBG_PRINTF("intv_min = %d,intv_max = %d,latency = %d,time_out = %d\r\n",
		cmd->intv_min,cmd->intv_max,cmd->latency,cmd->time_out);
	
    // Send the message
    ke_msg_send(cmd);
}

void appm_update_adv_data( uint8_t* adv_buff, uint8_t adv_len, uint8_t* scan_buff, uint8_t scan_len)
{
	if (ke_state_get(TASK_APP) == APPM_ADVERTISING 
                 && (adv_len <= ADV_DATA_LEN) && (scan_len <= ADV_DATA_LEN))
	{
		struct gapm_update_advertise_data_cmd *cmd =  KE_MSG_ALLOC(
            		            GAPM_UPDATE_ADVERTISE_DATA_CMD,
            		            TASK_GAPM,
            		            TASK_APP,
            		            gapm_update_advertise_data_cmd);

		cmd->operation = GAPM_UPDATE_ADVERTISE_DATA;
		cmd->adv_data_len = adv_len;
		cmd->scan_rsp_data_len = scan_len;

		//memcpy
		memcpy(&cmd->adv_data[0], adv_buff, adv_len);
		memcpy(&cmd->scan_rsp_data[0], scan_buff, scan_len);
        
		// Send the message
		ke_msg_send(cmd);
	}
}


uint8_t appm_get_dev_name(uint8_t* name)
{
    // copy name to provided pointer
    memcpy(name, app_env.dev_name, app_env.dev_name_len);
    // return name length
    return app_env.dev_name_len;
}

void appm_send_seurity_req(void)
{
	DBG_PRINTF("%s \r\n",__func__);
	app_sec_send_security_req(app_env.conidx);
}


//for LE test control

//HCI_SET_EVT_MASK_PAGE_2_CMD_OPCODE
void appm_set_evt_mask_page2(uint8_t* page2_event_mask)
{
		struct gapm_set_evt_msk_p2_cmd *cmd =  KE_MSG_ALLOC(
            		            GAPM_SET_EVT_MASK_P2_CMD,
            		            TASK_GAPM,
            		            TASK_APP,
            		            gapm_set_evt_msk_p2_cmd);
		cmd->operation = GAPM_SET_EVT_MSK_P2;
		memcpy(cmd->mask_p2_array,page2_event_mask,8);
		// Send the message
		ke_msg_send(cmd);
}
void appm_le_enh_rx_test_cmd(uint8_t rx_channel,uint8_t phy,uint8_t modulation_idx)
{
	struct gapm_enh_rx_test_cmd *cmd =  KE_MSG_ALLOC(
            		            GAPM_LE_ENH_RX_TEST_CMD,
            		            TASK_GAPM,
            		            TASK_APP,
            		            gapm_enh_rx_test_cmd);
	DBG_PRINTF("appm_le_enh_rx_test_cmd,snd GAPM_LE_ENH_RX_TEST_CMD msg\r\n");
	cmd->operation = GAPM_LE_ENH_RX_TEST;
	cmd->rx_channel = rx_channel;
	cmd->phy = phy;
	cmd->modulation_idx = modulation_idx;
	// Send the message
	ke_msg_send(cmd);
}
void appm_le_enh_tx_test_cmd(uint8_t tx_channel,uint8_t test_data_length,uint8_t packet_payload,uint8_t phy)
{
	struct gapm_enh_tx_test_cmd *cmd =  KE_MSG_ALLOC(
            		            GAPM_LE_ENH_TX_TEST_CMD,
            		            TASK_GAPM,
            		            TASK_APP,
            		            gapm_enh_tx_test_cmd);
	cmd->operation = GAPM_LE_ENH_TX_TEST;
	cmd->tx_channel = tx_channel;
	//N = (F-2402) / 2
	//Range: 0x00 to 0x27.
	//Frequency Range: 2402 MHz to 2480 MHz
	cmd->test_data_length = test_data_length;
	//0x00 to 0xFF
	cmd->packet_payload = packet_payload;
	//0x00 PRBS9 sequence '11111111100000111101? (in transmission order) as
	//described in [Vol 6] Part F, Section 4.1.5
	//0x01 Repeated '11110000' (in transmission order) sequence as described in
	//[Vol 6] Part F, Section 4.1.5
	//0x02 Repeated '10101010' (in transmission order) sequence as described in
	//[Vol 6] Part F, Section 4.1.5
	//0x03 PRBS15 sequence as described in [Vol 6] Part F, Section 4.1.5
	//0x04 Repeated '11111111' (in transmission order) sequence
	//0x05 Repeated '00000000' (in transmission order) sequence
	//0x06 Repeated '00001111' (in transmission order) sequence
	//0x07 Repeated '01010101' (in transmission order) sequence
	cmd->phy = phy;
	//0x01 Transmitter set to use the LE 1M PHY
	//0x02 Transmitter set to use the LE 2M PHY
	//0x03 Transmitter set to use the LE Coded PHY with S=8 data coding
	//0x04 Transmitter set to use the LE Coded PHY with S=2 data coding
	// Send the message
	ke_msg_send(cmd);
}
void appm_le_tx_test_cmd(uint8_t tx_channel,uint8_t test_data_length,uint8_t packet_payload)
{
	struct gapm_tx_test_cmd *cmd =  KE_MSG_ALLOC(
            		            GAPM_LE_TX_TEST_CMD,
            		            TASK_GAPM,
            		            TASK_APP,
            		            gapm_tx_test_cmd);
	DBG_PRINTF("appm_le_tx_test_cmd,snd GAPM_LE_TX_TEST_CMD msg\r\n");
	cmd->operation = GAPM_LE_TX_TEST;
	cmd->tx_channel = tx_channel;
	//N = (F-2402) / 2
	//Range: 0x00 to 0x27.
	//Frequency Range: 2402 MHz to 2480 MHz
	cmd->test_data_length = test_data_length;
	//0x00 to 0xFF
	cmd->packet_payload = packet_payload;
	ke_msg_send(cmd);
}
void appm_le_rx_test_cmd(uint8_t rx_channel)
{
	struct gapm_rx_test_cmd *cmd =  KE_MSG_ALLOC(
            		            GAPM_LE_RX_TEST_CMD,
            		            TASK_GAPM,
            		            TASK_APP,
            		            gapm_rx_test_cmd);
	DBG_PRINTF("appm_le_rx_test_cmd,snd GAPM_LE_RX_TEST_CMD msg\r\n");
	cmd->operation = GAPM_LE_RX_TEST;
	cmd->rx_channel = rx_channel;
	// Send the message
	ke_msg_send(cmd);
}
void appm_test_end_cmd(void)
{
	struct gapm_enh_tx_test_cmd *cmd =  KE_MSG_ALLOC(
            		            GAPM_LE_TEST_END_CMD,
            		            TASK_GAPM,
            		            TASK_APP,
            		            gapm_enh_tx_test_cmd);
	cmd->operation = GAPM_LE_TEST_END;
	// Send the message
	ke_msg_send(cmd);
}
#endif //(BLE_APP_PRESENT)

/// @} APP


