/**
 ****************************************************************************************
 *
 * @file uart.c
 *
 * @brief UART driver
 *
 * Copyright (C) RivieraWaves 2009-2015
 *
 *
 ****************************************************************************************
 */

/**
 ****************************************************************************************
 * @addtogroup UART
 * @{
 ****************************************************************************************
 */ 
/*
 * INCLUDE FILES
 ****************************************************************************************
 */
#include <stddef.h>     // standard definition
#include "timer.h"      // timer definition
#include "uart.h"       // uart definition
#include "BK3432_reg.h"
#include "reg_uart.h"   // uart register
#include "rwip.h"       // SW interface
#include "h4tl.h"
#if (NVDS_SUPPORT)
#include "nvds.h"                    // NVDS Definitions
#endif
#include "dbg.h"
 
#include "user_config.h"
  #if(BQB_TEST_ENABLE == 1 && STACK_SEL != OAD_STACK2 )
	#include "app_pts_test.h"
  #endif
/*
 * DEFINES
 *****************************************************************************************
 */

/// Max baudrate supported by this UART (in bps)
#define UART_BAUD_MAX      		  3500000
/// Min baudrate supported by this UART (in bps)
#define UART_BAUD_MIN      		  9600

/// Duration of 1 byte transfer over UART (10 bits) in us (for 921600 default baudrate)
#define UART_CHAR_DURATION        11

/*
 * ENUMERATION DEFINITIONS
 *****************************************************************************************
 */
 

 
 /*
 * common.c
 *
 *  Created on: 2013-5-7
 *  Author: pujie
 */

#include  <stdarg.h>
#include  <stdio.h>
#include "BK3432_reg.h"
#include "ea.h"
#include "BK_HCI_Protocol.h"
#include "app.h"
#include "llm.h"
#define Uart_Write_Byte(v)               (REG_APB3_UART_PORT=v)

#define UART_TX_FIFO_COUNT               (REG_APB3_UART_FIFO_STAT&0xff)
#define UART_RX_FIFO_COUNT               ((REG_APB3_UART_FIFO_STAT>>8)&0xff)
#define UART_TX_FIFO_FULL                (REG_APB3_UART_FIFO_STAT&0x00010000)
#define UART_TX_FIFO_EMPTY               (REG_APB3_UART_FIFO_STAT&0x00020000)
#define UART_RX_FIFO_FULL                (REG_APB3_UART_FIFO_STAT&0x00040000)
#define UART_RX_FIFO_EMPTY               (REG_APB3_UART_FIFO_STAT&0x00080000) 
#define UART_TX_WRITE_READY              (REG_APB3_UART_FIFO_STAT&0x00100000) 
#define UART_RX_READ_READY               (REG_APB3_UART_FIFO_STAT&0x00200000) 
#define bit_UART_TXFIFO_NEED_WRITE        0x01
#define bit_UART_RXFIFO_NEED_READ         0x02
#define bit_UART_RXFIFO_OVER_FLOW         0x04
#define bit_UART_RX_PARITY_ERROR          0x08
#define bit_UART_RX_STOP_ERROR            0x10
#define bit_UART_TX_PACKET_END            0x20
#define bit_UART_RX_PACKET_END            0x40
#define bit_UART_RXD_WAKEUP_DETECT        0x80


#define uart_tx_en    0x1      // 0: Disable TX, 1: Enable TX 
#define uart_rx_en    0x1      // 0: Disable RX, 1: Enable RX
#define irda_mode     0x0      // 0: UART  MODE, 1: IRDA MODE
#define data_len      0x3      // 0: 5 bits, 1: 6 bits, 2: 7 bits, 3: 8 bits
#define parity_en     0x0      // 0: NO Parity, 1: Enable Parity
#define parity_mode   0x1      // 0: Odd Check, 1: Even Check 
#define stop_bits     0x0      // 0: 1 stop-bit, 1: 2 stop-bit 
#define uart_clks     16000000 // UART's Main-Freq, 26M 
#define baud_rate     115200   // UART's Baud-Rate,  1M 


unsigned char uart_rx_buf[UART_RX_FIFO_MAX_COUNT];
unsigned char uart_tx_buf[UART_TX_FIFO_MAX_COUNT];
volatile bool uart_rx_done = 0;
volatile unsigned long uart_rx_index = 0;
uint8_t cur_read_buf_idx = 0;


#define UART1_READ_BYTE()                 ((REG_APB3_UART_PORT>>8)&0xff)
#define UART2_READ_BYTE()                 ((REG_APB3_UART2_PORT>>8)&0xff)


///UART Character format
enum UART_CHARFORMAT
{
    UART_CHARFORMAT_8 = 0,
    UART_CHARFORMAT_7 = 1
};

///UART Stop bit
enum UART_STOPBITS
{
    UART_STOPBITS_1 = 0,
    UART_STOPBITS_2 = 1  /* Note: The number of stop bits is 1.5 if a character format
                            with 5 bit is chosen*/
};

///UART Parity enable
enum UART_PARITY
{
    UART_PARITY_DISABLED = 0,
    UART_PARITY_ENABLED  = 1
};

///UART Parity type
enum UART_PARITYBIT
{
    UART_PARITYBIT_EVEN  = 0,
    UART_PARITYBIT_ODD   = 1,
    UART_PARITYBIT_SPACE = 2, // The parity bit is always 0.
    UART_PARITYBIT_MARK  = 3  // The parity bit is always 1.
};

///UART HW flow control
enum UART_HW_FLOW_CNTL
{
    UART_HW_FLOW_CNTL_DISABLED = 0,
    UART_HW_FLOW_CNTL_ENABLED = 1
};

///UART Input clock select
enum UART_INPUT_CLK_SEL
{
    UART_INPUT_CLK_SEL_0 = 0,
    UART_INPUT_CLK_SEL_1 = 1,
    UART_INPUT_CLK_SEL_2 = 2,
    UART_INPUT_CLK_SEL_3 = 3
};

///UART Interrupt enable/disable
enum UART_INT
{
    UART_INT_DISABLE = 0,
    UART_INT_ENABLE = 1
};

///UART Error detection
enum UART_ERROR_DETECT
{
    UART_ERROR_DETECT_DISABLED = 0,
    UART_ERROR_DETECT_ENABLED  = 1
};

/*
 * STRUCT DEFINITIONS
 *****************************************************************************************
 */
/* TX and RX channel class holding data used for asynchronous read and write data
 * transactions
 */
/// UART TX RX Channel
struct uart_txrxchannel
{
    /// call back function pointer
    void (*callback) (void*, uint8_t);
    /// Dummy data pointer returned to callback when operation is over.
    void* dummy;
};

/// UART environment structure
struct uart_env_tag
{
    /// tx channel
    struct uart_txrxchannel tx;
    /// rx channel
    struct uart_txrxchannel rx;
    /// error detect
    uint8_t errordetect;
    /// external wakeup
    bool ext_wakeup;
};

/*
 * GLOBAL VARIABLE DEFINITIONS
 ****************************************************************************************
 */

/// uart environment structure
struct uart_env_tag uart_env;
char uart_buff[128];



static UART_RX_CALLBACK_T usrt_rx_cb = NULL; 

/*
 * LOCAL FUNCTION DEFINITIONS
 ****************************************************************************************
 */


/*
 * EXPORTED FUNCTION DEFINITIONS
 ****************************************************************************************
 */
uint8_t Read_Uart_Buf(void)
{
	return uart_rx_buf[cur_read_buf_idx++];
}	

uint8_t Uart1_Read_Byte(void)
{
	return (REG_APB3_UART_PORT&0xff);
}	
uint8_t Uart2_Read_Byte(void)
{
	return (REG_APB3_UART2_PORT&0xff);
}	

int uart_putchar(char * st)
{
	uint8_t num = 0;
    while (*st) 
    {
		if(UART_TX_WRITE_READY)
		{
			REG_APB3_UART_PORT = *st;
	    	st++;
			num++;
	    }
	} 
    return num;
}
//EBQ add
int uart2_putchar(char * st)
{
	  uint8_t num = 0;
#if 1
    while (*st) 
    {
		if(UART2_TX_WRITE_READY)
		{
			REG_APB3_UART2_PORT = *st;
	    st++;
			num++;
	    }
	} 
#endif 		
    return num;
}
int uart2_printf(const char *fmt,...)
{

		int n;	

		va_list ap;
    va_start(ap, fmt);
    n=vsprintf(uart_buff, fmt, ap);
    va_end(ap);
    uart2_putchar(uart_buff);
		if(n > sizeof(uart_buff))
		{
			uart2_putchar("buff full \r\n");
		}

		
    return n;
}
int uart1_printf(const char *fmt,...)
{ 
	int n;	

	va_list ap;
	va_start(ap, fmt);
	n=vsprintf(uart_buff, fmt, ap);
	va_end(ap);
    uart_putchar(uart_buff);
	if(n > sizeof(uart_buff))
	{
		uart_putchar("buff full \r\n");
	}
		
    return n;
}

int uart_printf_null(const char *fmt,...)
{
	return 0;
}


char *hex2Str( uint8_t data)
{

  char hex[] = "0123456789ABCDEF";
  static char str[3];
  char *pStr = str;
 
  *pStr++ = hex[data >> 4];
  *pStr++ = hex[data & 0x0F];
  *pStr = 0;

  return str;
}

void uart_print_int(unsigned int num)
{
	uint8_t i;
	uint8_t m;

	uart_putchar((char *)"0x");
	for (i = 4;i > 0;i--)
	{
		m = ((num >> (8 * (i - 1)))& 0xff);
		uart_putchar(hex2Str(m));
	}
	uart_putchar("\r\n");
}



void cpu_delay( volatile unsigned int times)
{
    while(times--)
	{
		for(uint32_t i = 0;i < 1000;i++)
			;
	}				
}
/*
 * EXPORTED FUNCTION DEFINITIONS
 ****************************************************************************************
 */

//EBQ add 
void uart2_init(uint32_t baudrate)
{
  unsigned int baud_divisor ;
	
  REG_AHB0_ICU_UARTCLKCON   &= ~(0x1 << 0) ;  // Enable Uart's Clocks   
  switch(baudrate)
 {
 case 9600:
 case 19200:
 case 38400:
 case 57600:
 case 115200:
  baud_divisor = 0x89;
  break;
 case 921600:
  baud_divisor = 0x10;
  break;
 ////Pochin
 case 1000000:
 case 2000000:
  baud_divisor = (16000000/baudrate) - 1;
  break;
 default:
  baud_divisor = 0x89;
  break;
 }  
  
 
  REG_APB3_UART2_CFG  = (baud_divisor<<8) + 
                          (stop_bits   <<7) + 
                          (data_len    <<3) + 
                          (irda_mode   <<2) + 
                          (uart_rx_en  <<1) + 
                           uart_tx_en       ;
    //REG_APB3_UART2_FIFO_CFG  = 0x00031001 ;  // Set Fifo threshold 32
		REG_APB3_UART2_FIFO_CFG = (1<<BIT_TX_FIFO_THRESHOLD)|(16<<BIT_RX_FIFO_THRESHOLD)|(0x0 << BIT_STOP_DETECT_TIME);
		REG_APB3_UART2_INT_ENABLE = ((0x01 << 1) | (0x01 << 6) | (0x01 << 7));  //need read / stop end /rxd wake up
    REG_APB3_UART2_FLOW_CFG  = 0x00000000 ;  // No Flow Control   
    REG_APB3_UART2_WAKE_CFG  =  ((0x01 << 0 )| (0x01 << 20) |  (0x01 << 21)| (0x01 << 22));  // No Wake Control    

		REG_APB5_GPIOB_CFG  &= ~((0xC0<<BIT_GPIO_PULL_UP)  + (0xC0<<BIT_GPIO_PERI_EN));
		REG_APB5_GPIOB_CFG  |= ((0xC0<<BIT_GPIO_PULL_UP)); 
		REG_APB5_GPIOB_CFG  |=   (0xC0<<BIT_GPIO_OUT_EN_N);
		REG_APB5_GPIOB_DATA &= ~ (0xC0<<BIT_GPIO_INPUT_EN);
		
		uart_env.rx.callback = NULL;
    uart_env.rx.dummy    = NULL;
		REG_AHB0_ICU_INT_ENABLE |= (0x01 << 16);
    
}

void uart_init(uint32_t baudrate)
{
    unsigned int baud_divisor ;
    REG_AHB0_ICU_UARTCLKCON   &= ~(0x1 << 0) ;  // Enable Uart's Clocks 
    switch(baudrate)
 {
 case 9600:
 case 19200:
 case 38400:
 case 57600:
 case 115200:
  baud_divisor = 0x89;
  break;
 case 921600:
  baud_divisor = 0x10;
  break;
 ////Pochin
 case 1000000:
 case 2000000:
  baud_divisor = (16000000/baudrate) - 1;
  break;
 default:
  baud_divisor = 0x89;
  break;
 }  
    REG_APB3_UART_CFG  = (baud_divisor<<8) + 
                          (stop_bits   <<7) + 
                          (data_len    <<3) + 
                          (irda_mode   <<2) + 
                          (uart_rx_en  <<1) + 
                           uart_tx_en ;
  
    REG_APB3_UART_FIFO_CFG = (1<<BIT_TX_FIFO_THRESHOLD)|(16<<BIT_RX_FIFO_THRESHOLD)|(0x0 << BIT_STOP_DETECT_TIME);
		REG_APB3_UART_INT_ENABLE = ((0x01 << 1) | (0x01 << 6) | (0x01 << 7)); 
    REG_APB3_UART_FLOW_CFG  = 0x00000000 ;  // No Flow Control   
    REG_APB3_UART_WAKE_CFG  = ((0x01 << 0 )| (0x01 << 20) |  (0x01 << 21)| (0x01 << 22));  // No Wake Control
    	
    REG_APB5_GPIOA_CFG  &= ~((0x3<<BIT_GPIO_PULL_UP)  + (0x3<<BIT_GPIO_PERI_EN)); 
    REG_APB5_GPIOA_CFG  |= ((0x3<<BIT_GPIO_PULL_UP)); 
    REG_APB5_GPIOA_CFG  |=   (0x3<<BIT_GPIO_OUT_EN_N);
    REG_APB5_GPIOA_DATA &= ~ (0x3<<BIT_GPIO_INPUT_EN);
		

    uart_env.rx.callback = NULL;
    uart_env.rx.dummy    = NULL;

		REG_AHB0_ICU_INT_ENABLE |= (0x01 << 5);

}	



void uart_flow_on(void)
{
    // Configure modem (HW flow control enable)
    // uart_flow_en_setf(0);
}

void uart_clear_rxfifo(void)
{
	
	while(uart_rx_fifo_rd_ready_getf())
	{
		Uart1_Read_Byte();
	}
	memset(uart_rx_buf,0,UART_RX_FIFO_MAX_COUNT);
	
}
void uart2_clear_rxfifo(void)
{
	while(uart2_rx_fifo_rd_ready_getf())
	{
		Uart2_Read_Byte();
	}
	memset(uart_rx_buf,0,UART_RX_FIFO_MAX_COUNT);
	
}
bool uart_flow_off(void)
{
	
	  return true;
	
}

void uart_finish_transfers(void)
{
    uart_flow_en_setf(1);

    // Wait TX FIFO empty
    while(!uart_tx_fifo_empty_getf());
}
void uart2_finish_transfers(void)
{
    uart2_flow_en_setf(1);

    // Wait TX FIFO empty
    while(!uart2_tx_fifo_empty_getf());
}

void uart_read(uint8_t *bufptr, uint32_t size, void (*callback) (void*, uint8_t), void* dummy)
{
    // Sanity check
    ASSERT_ERR(bufptr != NULL);
    ASSERT_ERR(size != 0);
    ASSERT_ERR(callback != NULL);
    uart_env.rx.callback = callback;

    uart_env.rx.dummy    = dummy;
	
}
void uart2_read(uint8_t *bufptr, uint32_t size, void (*callback) (void*, uint8_t), void* dummy)
{
    // Sanity check
    ASSERT_ERR(bufptr != NULL);
    ASSERT_ERR(size != 0);
    ASSERT_ERR(callback != NULL);
    uart_env.rx.callback = callback;

    uart_env.rx.dummy    = dummy;
	
}


void uart_write(uint8_t *bufptr, uint32_t size, void (*callback) (void*, uint8_t), void* dummy)
{
	// Sanity check
	ASSERT_ERR(bufptr != NULL);
	ASSERT_ERR(size != 0);
	ASSERT_ERR(callback != NULL);
	
	uint8_t len;
	len = size;
		
	uart_env.tx.callback = callback;
	uart_env.tx.dummy    = dummy;
	
	//Delay_ms(100);
	while(len--)
	{
		//cpu_delay(10);
		if(UART_TX_WRITE_READY)
		{
			REG_APB3_UART_PORT = *bufptr;
        	bufptr++;
    	}
	}
		
	if(callback != NULL)
	{
		// Clear callback pointer
		uart_env.tx.callback = NULL;
		uart_env.tx.dummy    = NULL;
		// Call handler
		callback(dummy, RWIP_EIF_STATUS_OK);
	}
}
void uart2_write(uint8_t *bufptr, uint32_t size, void (*callback) (void*, uint8_t), void* dummy)
{
	// Sanity check
	ASSERT_ERR(bufptr != NULL);
	ASSERT_ERR(size != 0);
	ASSERT_ERR(callback != NULL);
	
	uint8_t len;
	len = size;
		
	uart_env.tx.callback = callback;
	uart_env.tx.dummy    = dummy;
	
	//Delay_ms(100);
	while(len--)
	{
		//cpu_delay(10);
		if(UART2_TX_WRITE_READY)
		{
			REG_APB3_UART2_PORT = *bufptr;
        	bufptr++;
    	}
	}
		
	if(callback != NULL)
	{
		// Clear callback pointer
		uart_env.tx.callback = NULL;
		uart_env.tx.dummy    = NULL;
		// Call handler
		callback(dummy, RWIP_EIF_STATUS_OK);
	}
}
static void uart_send_byte(unsigned char data)
{
	while (!uart_tx_fifo_empty_getf());
    
	REG_APB3_UART_PORT = data ;
}
static void uart2_send_byte(unsigned char data)
{
	while (!uart2_tx_fifo_empty_getf());
    
	REG_APB3_UART2_PORT = data ;
}
void uart_send(unsigned char *buff, int len)
{
    while (len--)
        uart_send_byte(*buff++);
}
void uart2_send(unsigned char *buff, int len)
{
    while (len--)
        uart2_send_byte(*buff++);
}
void uart_cb_register(UART_RX_CALLBACK_T cb)
{
	if(cb)
	{
		usrt_rx_cb = cb;
	}
}

void uart_cb_clear(void)
{
	if(usrt_rx_cb)
	{
		usrt_rx_cb = NULL;
	}
}

volatile uint8_t uart_dut_reg_flag = 0;
uint16_t uart_recv_total_len = 0;
uint8_t uart_buf_parser_finish = 1;
uint8_t need_to_clr_buf = 0;
#if (UART_RX_GAP_MODIFY)
#include "ke_timer.h"             // Kernel timer
#include "app_task.h"
#define IDLE_STATE  0
#define TYPE_STATE  1
#define OPCODE1_STATE 2
#define OPCODE2_STATE 3
#define DATA_HANDLE_L 4
#define PRAM_LEN_STATE 5
#define DATA_HANDLE_H 6
#define ACL_DATA_TOTAL_LEN_L_STATE 7
#define ACL_DATA_TOTAL_LEN_H_STATE 8
#define PAYLOAD_STATE 9

#define CMD_TYPE_TAG 1
#define DATA_TYPE_TAG 2

//uint16_t uart_recv_total_len = 0;
extern uint8_t rx_to;
uint16_t total_len;
uint8_t  uart_state;
uint8_t  packet_type;
uint8_t  uart_hci_recv_state = TYPE_STATE;
uint8_t  paser_active = 1;

#define  BK_PRINTF DBG_PRINTF
#endif

void uart2_isr(void)
{		
    uint32_t IntStat;
		
		//extern uint8_t uart_buf_parser_finish;//merge
		#if BLE_TESTER
		//void (*callback) (void*, uint8_t) = NULL;
		//void* data =NULL;
		#endif
	
	
			IntStat = uart2_isr_stat_get();	
			if(uart2_rx_fifo_need_rd_isr_getf() || uart2_rx_end_isr_getf()|| uart2_rxd_wakeup_isr_getf())
			{
				while((REG_APB3_UART2_FIFO_STAT & (0x01 << 21)))
				{
					uart_rx_buf[uart_rx_index++] = UART2_READ_BYTE();
					if( UART_RX_FIFO_MAX_COUNT == uart_rx_index )
					{
						uart_rx_index = 0;
					}
				}

				uart_rx_done = 1;	
				#if 0
				if(usrt_rx_cb)
				{
					(*usrt_rx_cb)(uart_rx_buf, uart_rx_index);
				}
				#endif
				//uart_rx_index = 0;  
			}
			#if BLE_TESTER
				if( uart2_rx_end_isr_getf())
				{
					DBG_PRINTF("UR2:Done:%d,[%x:%x:%x]\r\n",uart_rx_index,uart_rx_buf[0],uart_rx_buf[1],uart_rx_buf[2]);
						if((uart_rx_buf[0] == 0x01) &&(uart_rx_buf[1] == 0xe0)&& (uart_rx_buf[2] == 0xfc))
						{
								uart_dut_reg_flag = REG_MODE;
						}
						else if((uart_rx_buf[0] == 0x01) && (uart_rx_index > 3))
						{
								DBG_PRINTF("UR2:##LE_CMD\r\n");
								uart_dut_reg_flag = DUT_MODE;
						}
						else if((uart_rx_buf[0] == 0x02) && (uart_rx_index > 3))
						{
							//Beken_Gpio_Targ(GPIOD_2);//llc_le_con_cmp_evt_send
							uart_dut_reg_flag = LE_DATA_MODE;
							//BK_PRINTF("##LE_DATA\r\n");
					//BK_PRINTF("Done:%d,[%x:%x:%x:%x:%x]\r\n",uart_rx_index,uart_rx_buf[0],uart_rx_buf[1],uart_rx_buf[2],uart_rx_buf[3],uart_rx_buf[4]);
						}
						else
						{
		//				uint8_t i;
		//				for(i=0;i < uart_rx_index;i++)
		//				{
		//					BK_PRINTF("%x:",uart_rx_buf[i]);	
		//				}
		//				BK_PRINTF("\r\n");	
						}
						if(uart_dut_reg_flag != 0)
						{
							uart_recv_total_len = uart_rx_index;
							uart_buf_parser_finish = 0;
							if(uart_dut_reg_flag == LE_DATA_MODE)
							{
								//BK_PRINTF("uart_recv_total_lenD'(%d)\r\n",uart_recv_total_len);
							}
					
								//BK_PRINTF("set uart_rx_done(%x)\r\n",uart_dut_reg_flag);
								DBG_PRINTF("UR2:uart_rx_done!!!\r\n");
								uart_rx_done = 1 ;
						}
						else
						{
								DBG_PRINTF("UR2:clr uart_rx_index\r\n");
								uart_rx_index = 0;
						}
				
				}
			#endif // BLE_TESTER
				DBG_PRINTF("UR2:(br)uart2_isr_stat(%x)\r\n",IntStat);
			uart2_isr_stat_set(IntStat);
}
//void uart_isr(void)
void uart1_isr(void)
{		
    uint32_t IntStat;
		
    //extern uint8_t uart_buf_parser_finish;//merge
    #if BLE_TESTER
    //void (*callback) (void*, uint8_t) = NULL;
    //void* data =NULL;
    #endif		
		
	IntStat = uart_isr_stat_get();	 

    #if (UART_RX_GAP_MODIFY)       
    if(uart_rx_fifo_need_rd_isr_getf() || uart_rx_end_isr_getf()|| uart_rxd_wakeup_isr_getf())
	{
        while((REG_APB3_UART_FIFO_STAT & (0x01 << 21)))
        {   
        uint8_t data;
            data = uart_rx_buf[uart_rx_index++] = UART1_READ_BYTE();
            if(uart_hci_recv_state == TYPE_STATE)//0
            {
                packet_type = data;
                switch(packet_type)
                {
                    case CMD_TYPE_TAG :
                            uart_hci_recv_state = OPCODE1_STATE;
                            BK_PRINTF("pkt_typ(%x)\r\n",data);
                    break;
                    case DATA_TYPE_TAG :
                            uart_hci_recv_state = DATA_HANDLE_L;
                            BK_PRINTF("pkt_typ(%x)\r\n",data);
                    break;
                }
            }
            else
            if(uart_hci_recv_state == DATA_HANDLE_L)
            {
                uart_hci_recv_state = DATA_HANDLE_H;
                BK_PRINTF("hdl_L(%x)\r\n",data);
            }
            else
            if(uart_hci_recv_state == DATA_HANDLE_H)
            {
                BK_PRINTF("hdl_H(%x)\r\n",data);
                uart_hci_recv_state = ACL_DATA_TOTAL_LEN_L_STATE;
                
            }
            else 
            if(uart_hci_recv_state == ACL_DATA_TOTAL_LEN_L_STATE)
            {
                total_len |= data;
                uart_hci_recv_state = ACL_DATA_TOTAL_LEN_H_STATE;
                BK_PRINTF("LEN_L(%x)\r\n",data);
            }
            else if(uart_hci_recv_state == ACL_DATA_TOTAL_LEN_H_STATE)
            {
                uint16_t  len;
                len = data;
                len <<= 8;
                total_len |= len;
                uart_hci_recv_state = PAYLOAD_STATE;
                BK_PRINTF("LEN_H(%x),total(%d)\r\n",data,total_len);
            }
            else
            if(uart_hci_recv_state == OPCODE1_STATE)//1
            {
                uart_hci_recv_state = OPCODE2_STATE;
                BK_PRINTF("op1(%x)\r\n",data);
            }
            else
            if(uart_hci_recv_state == OPCODE2_STATE)//2
            {
                uart_hci_recv_state = PRAM_LEN_STATE;
                BK_PRINTF("op2(%x)\r\n",data);
            }
            else
            if(uart_hci_recv_state == PRAM_LEN_STATE)//3
            {
                 total_len = uart_rx_buf[3];
                 BK_PRINTF("param_len(%x)\r\n",total_len);
                 uart_hci_recv_state = PAYLOAD_STATE;
                 BK_PRINTF("==> PAYLOAD_STATE\r\n");
            }
            else
            if(uart_hci_recv_state == PAYLOAD_STATE)
            {
                    if(total_len > 0)
                    {
                         total_len--;
                    }
            }
            ke_timer_set(UART_RX_TO_IND, TASK_APP, 4000);//4000*10ms = 40 sec
            if( UART_RX_FIFO_MAX_COUNT == uart_rx_index )
            {
                BK_PRINTF("fifo_full\r\n");
                uart_rx_index = 0;
                BK_PRINTF("2.clr uart_rx_index\r\n");
            }   
        }
    }    
		//BK_PRINTF("\r\n");
    //UART_PRINTF("TRAhcit_UART_Rx,uart_rx_done = %x,uart_rx_index = %x\r\n",uart_rx_done,uart_rx_index);
    //BK_PRINTF("uart_rx_index(%d),uart_rx_done(%x)\r\n",uart_rx_index,uart_rx_done);
    if( uart_rx_end_isr_getf())
    {
        if((total_len == 0) && (uart_hci_recv_state == PAYLOAD_STATE)) 
        {
            if((uart_rx_buf[0] == 0x01) &&(uart_rx_buf[1] == 0xe0)&& (uart_rx_buf[2] == 0xfc))
            {
                    uart_dut_reg_flag = REG_MODE;
            }
            //else if((uart_rx_buf[0] == 0x01) && (uart_rx_index > 3))
            else if(packet_type == CMD_TYPE_TAG)
            {
                    //BK_PRINTF("##LE_CMD\r\n");
                    uart_dut_reg_flag = DUT_MODE;
            }
            //else if((uart_rx_buf[0] == 0x02) && (uart_rx_index > 3))
            else if(packet_type == DATA_TYPE_TAG)
            {
                    //Beken_Gpio_Targ(GPIOD_2);//llc_le_con_cmp_evt_send
                    //BK_PRINTF("##LE_DAT\r\n");
                    uart_dut_reg_flag = LE_DATA_MODE;
                //BK_PRINTF("##LE_DATA\r\n");
                //BK_PRINTF("Done:%d,[%x:%x:%x:%x:%x]\r\n",uart_rx_index,uart_rx_buf[0],uart_rx_buf[1],uart_rx_buf[2],uart_rx_buf[3],uart_rx_buf[4]);
            }
            if(uart_dut_reg_flag !=0)
            {
                    uart_recv_total_len = uart_rx_index;
                    uart_buf_parser_finish = 0;
                    uart_hci_recv_state = TYPE_STATE;
                    ke_timer_clear(UART_RX_TO_IND, TASK_APP);
                  uart_rx_done = 1;
                  BK_PRINTF("##uart_rx_done\r\n");
            }
        }
        else
        {
            //BK_PRINTF("cont1 wait(%x)!!!\r\n",total_len);
            if(rx_to == 1)
            {
                uart_rx_index = 0;
                BK_PRINTF("rx timeout\r\n");
            }
        }
    }	    
    
    #else  // BLE_TESTER  
    if(uart_rx_fifo_need_rd_isr_getf() || uart_rx_end_isr_getf()|| uart_rxd_wakeup_isr_getf())
	{        
        while((REG_APB3_UART_FIFO_STAT & (0x01 << 21)))
        {
            uart_rx_buf[uart_rx_index++] = UART1_READ_BYTE();
            if( UART_RX_FIFO_MAX_COUNT == uart_rx_index )
            {
                uart_rx_index = 0;
            }
        }
        uart_rx_done = 1;	                
        #if 0
        if(usrt_rx_cb)
        {
            (*usrt_rx_cb)(uart_rx_buf, uart_rx_index);
        }
        #endif
        //uart_rx_index = 0;  
    }
      #if BLE_TESTER
    if( uart_rx_end_isr_getf())
    {
        DBG_PRINTF("UR1:Done:%d,[%x:%x:%x]\r\n",uart_rx_index,uart_rx_buf[0],uart_rx_buf[1],uart_rx_buf[2]);
        if((uart_rx_buf[0] == 0x01) &&(uart_rx_buf[1] == 0xe0)&& (uart_rx_buf[2] == 0xfc))
        {
            uart_dut_reg_flag = REG_MODE;
        }
        else if((uart_rx_buf[0] == 0x01) && (uart_rx_index > 3))
        {
            DBG_PRINTF("UR1:##LE_CMD\r\n");
            uart_dut_reg_flag = DUT_MODE;
        }
        else if((uart_rx_buf[0] == 0x02) && (uart_rx_index > 3))
        {
            //Beken_Gpio_Targ(GPIOD_2);//llc_le_con_cmp_evt_send
            uart_dut_reg_flag = LE_DATA_MODE;
            //BK_PRINTF("##LE_DATA\r\n");
            //BK_PRINTF("Done:%d,[%x:%x:%x:%x:%x]\r\n",uart_rx_index,uart_rx_buf[0],uart_rx_buf[1],uart_rx_buf[2],uart_rx_buf[3],uart_rx_buf[4]);
        }
        else
        {
//		        uint8_t i;
//				for(i=0;i < uart_rx_index;i++)
//				{
//					BK_PRINTF("%x:",uart_rx_buf[i]);	
//				}
//				BK_PRINTF("\r\n");	
        }
        if(uart_dut_reg_flag != 0)
        {
            uart_recv_total_len = uart_rx_index;
            uart_buf_parser_finish = 0;
            if(uart_dut_reg_flag == LE_DATA_MODE)
            {
                //BK_PRINTF("uart_recv_total_lenD'(%d)\r\n",uart_recv_total_len);
            }
            //BK_PRINTF("set uart_rx_done(%x)\r\n",uart_dut_reg_flag);
            DBG_PRINTF("UR1:uart_rx_done!!!\r\n");
            uart_rx_done = 1 ;
        }
        else
        {
            DBG_PRINTF("UR1:3.clr uart_rx_index\r\n");
            uart_rx_index = 0;
        }        
    }  
      #endif // BLE_TESTER
    DBG_PRINTF("UR1:(br)uart_isr_stat_set(%x)\r\n",IntStat);             
    
	#endif 	//#if UART_RX_GAP_MODIFY 	 
    uart_isr_stat_set(IntStat);
}

uint8_t check_uart_stop(void)
{
	return uart_tx_fifo_empty_getf();
}
/////////////////////////////simple paser uart function 20210113/////////////////////
HCI_COMMAND_PACKET *pHCIrxBuf = (HCI_COMMAND_PACKET *)(&uart_rx_buf[0]);
HCI_EVENT_PACKET   *pHCItxBuf = (HCI_EVENT_PACKET *)(&uart_tx_buf[0]);
void clear_uart_buffer(void)
{
	//BK_PRINTF("1.clr uart_rx_index.rx_done(%x)\r\n",uart_rx_done);
    uart_rx_index = 0;
    uart_rx_done = 0;
    //memset(uart_rx_buf, 0, sizeof(uart_rx_buf)); /**< Clear the RX buffer */
    //memset(uart_tx_buf, 0, sizeof(uart_tx_buf)); /**< Clear the TX buffer */
	//BK_PRINTF("after clr,rx_done(%d)\r\n",uart_rx_done);
}
void TRAhcit_UART_Tx(uint8_t sel) 
{
	//  DBG_PRINTF("TRAhcit_UART_Tx\r\n");
    u32 tx_len       = HCI_EVENT_HEAD_LENGTH+pHCItxBuf->total;
    pHCItxBuf->code  = TRA_HCIT_EVENT;
    pHCItxBuf->event = HCI_COMMAND_COMPLETE_EVENT;
	  if(sel == 1)
		{
			uart_send(uart_tx_buf, tx_len);
		}
		else if(sel == 2)
		{
			uart2_send(uart_tx_buf, tx_len);	
		}
}
uint16_t tst_received_count = 0;
void Send_HCI_cmd_cmp_to_uart(uint16_t opcode,uint8_t status,uint8_t* input_param,uint8_t total_len)
{
			uint8_t i=0;
			pHCItxBuf->total = total_len;
			pHCItxBuf->param[0] = 5;
		  pHCItxBuf->param[1] = opcode;
		  pHCItxBuf->param[2] = opcode>>8;
		  pHCItxBuf->param[3] = status;//status
			if(total_len > 4)
			{	
				for(i=0;i<(total_len-4);i++)
				{
					pHCItxBuf->param[4+i] = input_param[i];
				}
			}
			TRAhcit_UART_Tx(CMD_UART_NO);//send command complete
}
uint8_t  HCI_paser_HCI_cmd(void)//return ,0xff not valid,0x01 finish
{
	uint8_t i;
	uint16_t opcode;
	extern uint8_t rf_test_en;
	if ( (pHCIrxBuf->code!= TRA_HCIT_COMMAND))
	{
			return (0xff);
	}
	DBG_PRINTF("Raw data\r\n");
	for(i=0;i<uart_recv_total_len;i++)
	{	
		DBG_PRINTF("%x:",uart_rx_buf[i]);
	}	
	DBG_PRINTF("\r\n");
	DBG_PRINTF("Display HCI cmd\r\n");
	opcode = pHCIrxBuf->opcode.ogf;
	opcode <<= 10;
	opcode |= pHCIrxBuf->opcode.ocf;
	DBG_PRINTF("opcode(%x),ocf(%x),ogf(%x)\r\n",opcode,pHCIrxBuf->opcode.ocf,pHCIrxBuf->opcode.ogf);
	DBG_PRINTF("cmd(%x)",pHCIrxBuf->cmd);
	DBG_PRINTF("total(%x)",pHCIrxBuf->total);
	
	switch(opcode)
	{
		case HCI_RESET_CMD_OPCODE ://cmd packet 01 03 0C 00
				DBG_PRINTF("HCI_RESET_CMD_OPCODE\r\n");
				rf_test_en = 1;
				appm_stop_advertising();
//				pHCItxBuf->total = 0x04;//param length
//				pHCItxBuf->param[0] = 5;
//				pHCItxBuf->param[1] = opcode;
//				pHCItxBuf->param[2] = opcode>>8;
//				pHCItxBuf->param[3] = 0;//status
				Send_HCI_cmd_cmp_to_uart(opcode,0,NULL,4);
				//send command complete 04 0E 04 05 03 0C 00
    break;
		case HCI_RD_BD_ADDR_CMD_OPCODE ://cmd packet 01 09 10 00
					DBG_PRINTF("HCI_RD_BD_ADDR_CMD_OPCODE\r\n");
                rf_test_en = 1;
//				pHCItxBuf->total = 0x0A;//param length
//				pHCItxBuf->param[0] = 5;
//				pHCItxBuf->param[1] = opcode;
//				pHCItxBuf->param[2] = opcode>>8;
//				pHCItxBuf->param[3] = 0;//status success
//				for(i=0;i<6;i++)
//				{
//					pHCItxBuf->param[4+i] = co_default_bdaddr.addr[i];
//				}
				Send_HCI_cmd_cmp_to_uart(opcode,0,&co_default_bdaddr.addr[0],0x0A);
				//send command complete 04 0E 0A 05 09 10 00 32 34 32 34 32 34 
		 break;
		case HCI_LE_RD_LOCAL_SUPP_FEATS_CMD_OPCODE : //cmd packet 01 03 20 00
		{
					 ///8-byte array for LE features
					uint8_t loc_feats[LE_FEATS_LEN];
					rf_test_en = 1;
					appm_stop_advertising();
			    memcpy(&loc_feats[0],&llm_local_le_feats.feats[0],LE_FEATS_LEN);
					loc_feats[3] |= BLE_REMOTE_PUBLIC_KEY_VALIDATION;//EBQ LL/CON/MAS/BV-13
				  Send_HCI_cmd_cmp_to_uart(opcode,0,&loc_feats[0],0x0C);
				//send command complete 04 0E 0C 05 03 20 00 3F 01 00 08 00 00 00 00 
		}
		 break;
		 case HCI_SET_EVT_MASK_CMD_OPCODE ://cmd packet 01 01 0C 08 90 88 00 00 00 80 00 20
		 {	
					DBG_PRINTF("HCI_SET_EVT_MASK_CMD_OPCODE\r\n");
//					pHCItxBuf->total = 0x04;//param length
//					pHCItxBuf->param[0] = 5;
//					pHCItxBuf->param[1] = opcode;
//					pHCItxBuf->param[2] = opcode>>8;
//					pHCItxBuf->param[3] = 0;//status
					Send_HCI_cmd_cmp_to_uart(opcode,0,NULL,4);
		 }
					//send command complete 04 0E 04 05 01 0C 00 
		 break;
		 case HCI_LE_SET_EVT_MASK_CMD_OPCODE ://cmd packet 01 01 20 08 FF 0F 00 00 00 00 00 00
		 {
					DBG_PRINTF("HCI_LE_SET_EVT_MASK_CMD_OPCODE\r\n");
//					pHCItxBuf->total = 0x04;//param length
//					pHCItxBuf->param[0] = 5;
//					pHCItxBuf->param[1] = opcode;
//					pHCItxBuf->param[2] = opcode>>8;
//					pHCItxBuf->param[3] = 0;//status
					Send_HCI_cmd_cmp_to_uart(opcode,0,NULL,4);
					//send command complete 04 0E 04 05 01 20 00
		 }
		 break;
		 case HCI_SET_EVT_MASK_PAGE_2_CMD_OPCODE ://cmd packet 01 63 0C 08 00 01 80 00 00 00 00 00
		 {
					uint8_t page2_mask_array[8];
					DBG_PRINTF("HCI_SET_EVT_MASK_PAGE_2_CMD_OPCODE\r\n");
			    memcpy(&page2_mask_array[0],&uart_rx_buf[4],8);
					appm_set_evt_mask_page2(&page2_mask_array[0]);
//					pHCItxBuf->total = 0x04;//param length
//					pHCItxBuf->param[0] = 5;
//					pHCItxBuf->param[1] = opcode;
//					pHCItxBuf->param[2] = opcode>>8;
//					pHCItxBuf->param[3] = 0;//status
					//send command complete 04 0E 04 05 63 0C 00
		 }
		 break;
		 case HCI_LE_ENH_RX_TEST_CMD_OPCODE : //cmd packet 01 33 20 03 00 01 00 (1M) or 01 33 20 03 00 02 00 (2M)
		//#RX_Channel,
		//N = (F-2402) / 2
		//Range: 0x00 to 0x27.
		//Frequency Range: 2402 MHz to 2480 MHz
		//#PHY,
		//0x01 Receiver set to use the LE 1M PHY
		//0x02 Receiver set to use the LE 2M PHY
		//0x03 Receiver set to use the LE Coded PHY
		//#Modulation_Index
		//0x00 Assume transmitter will have a standard modulation index
		//0x01 Assume transmitter will have a stable modulation index
//					pHCItxBuf->total = 0x04;//param length
//					pHCItxBuf->param[0] = 5;
//					pHCItxBuf->param[1] = opcode;
//					pHCItxBuf->param[2] = opcode>>8;
//					pHCItxBuf->param[3] = 0;//status
					//Send_HCI_cmd_cmp_to_uart(opcode,0,NULL,0);
					//send command complete 04 0E 04 05 33 20 00
			DBG_PRINTF("HCI_LE_ENH_RX_TEST_CMD_OPCODE\r\n");
		 appm_le_enh_rx_test_cmd(uart_rx_buf[4],uart_rx_buf[5],uart_rx_buf[6]);
		 break;
		 case HCI_LE_ENH_TX_TEST_CMD_OPCODE : //cmd packet 01 34 20 04 00 FF 00 02(2M) or 34 20 04 00 FF 00 01 (1M)
		//#TX_Channel,N = 0xXX
		//N = (F-2402) / 2
		//Range: 0x00 to 0x27.
		//Frequency Range: 2402 MHz to 2480 MHz
		//#Test_Data_Length,0x00 to 0xFF
		//#Packet_Payload,
		//0x00 PRBS9 sequence '11111111100000111101? (in transmission order) as
		//described in [Vol 6] Part F, Section 4.1.5
		//0x01 Repeated '11110000' (in transmission order) sequence as described in
		//[Vol 6] Part F, Section 4.1.5
		//0x02 Repeated '10101010' (in transmission order) sequence as described in
		//[Vol 6] Part F, Section 4.1.5
		//0x03 PRBS15 sequence as described in [Vol 6] Part F, Section 4.1.5
		//0x04 Repeated '11111111' (in transmission order) sequence
		//0x05 Repeated '00000000' (in transmission order) sequence
		//0x06 Repeated '00001111' (in transmission order) sequence
		//0x07 Repeated '01010101' (in transmission order) sequence
		//#PHY
		//0x01 Transmitter set to use the LE 1M PHY
		//0x02 Transmitter set to use the LE 2M PHY
		//0x03 Transmitter set to use the LE Coded PHY with S=8 data coding
		//0x04 Transmitter set to use the LE Coded PHY with S=2 data coding
//		pHCItxBuf->total = 0x04;//param length
//		pHCItxBuf->param[0] = 5;
//		pHCItxBuf->param[1] = opcode;
//		pHCItxBuf->param[2] = opcode>>8;
//		pHCItxBuf->param[3] = 0;//status
			DBG_PRINTF("HCI_LE_ENH_TX_TEST_CMD_OPCODE\r\n");
			appm_le_enh_tx_test_cmd(uart_rx_buf[4],uart_rx_buf[5],uart_rx_buf[6],uart_rx_buf[7]);
		//send command complete 04 0E 04 05 34 20 00
		 break;
		 case HCI_LE_RX_TEST_CMD_OPCODE ://cmd packet 01 1D 20 01 00 
		//RX_Channel,
		//N = (F-2402) / 2
		//Range: 0x00 to 0x27.
		//Frequency Range: 2402 MHz to 2480 MHz
//		pHCItxBuf->total = 0x04;//param length
//		pHCItxBuf->param[0] = 5;
//		pHCItxBuf->param[1] = opcode;
//		pHCItxBuf->param[2] = opcode>>8;
//		pHCItxBuf->param[3] = 0;//status
		 DBG_PRINTF("HCI_LE_RX_TEST_CMD_OPCODE\r\n");
		 appm_le_rx_test_cmd(uart_rx_buf[4]);
		//send command complete 04 0E 04 05 1D 20 00
		 break;
		 case HCI_LE_TX_TEST_CMD_OPCODE ://cmd packet 01 1E 20 03 00 FF 00
		//#TX_Channel,N = 0xXX
		//N = (F-2402) / 2
		//Range: 0x00 to 0x27.
		//Frequency Range: 2402 MHz to 2480 MHz
		//#Test_Data_Length,0x00 to 0xFF
		//#Packet_Payload,
		//0x00 PRBS9 sequence '11111111100000111101? (in transmission order) as
		//described in [Vol 6] Part F, Section 4.1.5
		//0x01 Repeated '11110000' (in transmission order) sequence as described in
		//[Vol 6] Part F, Section 4.1.5
		//0x02 Repeated '10101010' (in transmission order) sequence as described in
		//[Vol 6] Part F, Section 4.1.5
		//0x03 PRBS15 sequence as described in [Vol 6] Part F, Section 4.1.5
		//0x04 Repeated '11111111' (in transmission order) sequence
		//0x05 Repeated '00000000' (in transmission order) sequence
		//0x06 Repeated '00001111' (in transmission order) sequence
		//0x07 Repeated '01010101' (in transmission order) sequence
//		pHCItxBuf->total = 0x04;//param length
//		pHCItxBuf->param[0] = 5;
//		pHCItxBuf->param[1] = opcode;
//		pHCItxBuf->param[2] = opcode>>8;
//		pHCItxBuf->param[3] = 0;//status
			DBG_PRINTF("HCI_LE_TX_TEST_CMD_OPCODE\r\n");
			appm_le_tx_test_cmd(uart_rx_buf[4],uart_rx_buf[5],uart_rx_buf[6]);
		//send command complete 04 0E 04 05 1E 20 00
		 break;
		 case HCI_LE_TEST_END_CMD_OPCODE ://cmd packet 01 1F 20 00
			  DBG_PRINTF("HCI_LE_TEST_END_CMD_OPCODE\r\n");
				rf_test_en = 1;
				appm_stop_advertising();
		 //send command complete 04 0E 06 05 1F 20 00 D3 05
//			pHCItxBuf->total = 0x06;//param length
//			pHCItxBuf->param[0] = 5;
//			pHCItxBuf->param[1] = opcode;
//			pHCItxBuf->param[2] = opcode>>8;
//			pHCItxBuf->param[3] = 0;//status
//			pHCItxBuf->param[4] = tst_received_count;
//			pHCItxBuf->param[5] = tst_received_count>>8;
				appm_test_end_cmd();
		 break;
	#if(BQB_TEST_ENABLE == 1 && STACK_SEL != OAD_STACK2 )
		 case HCI_CUSTOMER_BQB_TEST_CMD_OPCODE:
			app_pts_test_copy_rx_data((uint8_t *)pHCIrxBuf, (pHCIrxBuf->total + 4));
		 break;
	#endif	 
         
#if 1         
        case HCI_DBG_TEST_BKTOOLS_CMD_OPCODE:     
        if(pHCIrxBuf->cmd == 0xfe)  //01 E0 FC 02 fe 0f 
        {            
            extern struct rwip_rf_api rwip_rf; 
            rwip_rf.txpwr_max = (pHCIrxBuf->param[0] & 0xf);
            DBG_PRINTF(" rwip_rf.txpwr_max=0x%x \r\n",rwip_rf.txpwr_max);      
        }
         break;   
#endif        
             
	}
	
	need_to_clr_buf = 1;
	return 1;
}
void TRAhcit_UART_Rx(void)
{
	extern uint8_t uart_buf_parser_finish;
	
	if(need_to_clr_buf == 1)
	{
		//BK_PRINTF("#clr buf!!!\r\n");
		need_to_clr_buf = 0;
		if(uart_buf_parser_finish == 0)
		{
			//BK_PRINTF("nf1(%x)\r\n",uart_rx_index);
			HCI_paser_HCI_cmd();
		}
		else
		{
			goto  ret;
		}
		
	}
  if ((uart_rx_done != 1) || (uart_rx_index == 0))
	{
        return;
	}
	

	uart_buf_parser_finish = HCI_paser_HCI_cmd();

//	if(uart_rx_done == 1)
//	{
//		return;
//	}
//	
//	
////loop_mode:
//    if (   (pHCIrxBuf->code       != TRA_HCIT_COMMAND)
//        || (pHCIrxBuf->opcode.ogf != VENDOR_SPECIFIC_DEBUG_OGF)
//        || (pHCIrxBuf->opcode.ocf != BEKEN_OCF)

//        || (uart_rx_index         != (HCI_COMMAND_HEAD_LENGTH+pHCIrxBuf->total))

//       )
//        goto ret;

    
  

	
ret:
	
    clear_uart_buffer();
}
/// @} UART
